@@ -1,4 +1,7 @@
-#----------------------------------------------------------------------------------------
+#-------------------------------------------------------------------------------
+# TO DO
+# i) Ardo residuals ??
+#-------------------------------------------------------------------------------
 gamlssNP<-function(formula,
                     random =~1,
                     family = NO(),
@@ -17,11 +20,11 @@ gamlssNP<-function(formula,
 ## R code originally by Ross Darnell (2002), modifications and extensions
 ## by Jochen Einbeck / John Hinde (2005).
 ## modified for gamlss by Mikis Stasinopoulos Thursday, July 20, 2006 
-#----------------------------------------------------------------------------------------
+#-------------------------------------------------------------------------------
 ## functions within gamlssNP
-#----------------------------------------------------------------------------------------
+#-------------------------------------------------------------------------------
 .gamlss.bi.list <- eval(quote(.gamlss.bi.list), envir = getNamespace("gamlss"))
-## expanding the data
+## expanding the data ----------------------------------------------------------
 expand.vc <- function(x,ni)
 {
   if (length(ni)==1)
@@ -43,8 +46,8 @@ expand.vc <- function(x,ni)
       xx
   }
 }
-#----------------------------------------------------------------------------------------
-#----------------------------------------------------------------------------------------
+#-------------------------------------------------------------------------------
+#-------------------------------------------------------------------------------
 weightslogl.calc.w <- function(p,fjk,weights){
   # p is a vector of length K containing the mixture proportions
   # fjk is a JXK matrix of log densities
@@ -56,8 +59,8 @@ weightslogl.calc.w <- function(p,fjk,weights){
   #l <- sum(log(Spf)) # log likelihood
   list(w=w,ML.dev=ML.dev)
 }
-#----------------------------------------------------------------------------------------
-#----------------------------------------------------------------------------------------
+#-------------------------------------------------------------------------------
+#-------------------------------------------------------------------------------
 get.log.likelihood.firstTime <- function(obj,mu, ...)
 {
 if (!is.gamlss(obj))  stop(paste("This is not an gamlss object", "\n", ""))
@@ -89,8 +92,8 @@ if (!is.gamlss(obj))  stop(paste("This is not an gamlss object", "\n", ""))
             else eval(call(dfun,x=yy, mu=mu, sigma=sigma, nu=nu, tau=tau,log=TRUE))})
 lik
 }
-#----------------------------------------------------------------------------------------
-#----------------------------------------------------------------------------------------
+#-------------------------------------------------------------------------------
+#-------------------------------------------------------------------------------
 get.log.likelihood <- function(obj, ...)
 {
 if (!is.gamlss(obj))  stop(paste("This is not an gamlss object", "\n", ""))
@@ -109,9 +112,9 @@ if (!is.gamlss(obj))  stop(paste("This is not an gamlss object", "\n", ""))
              else eval(call(dfun,x=obj$y, mu=fitted(obj,"mu"), sigma=fitted(obj,"sigma"), nu=fitted(obj,"nu"), tau=fitted(obj,"tau"),log=TRUE))})
 lik
 }                          
-#----------------------------------------------------------------------------------------
-#----------------------------------------------------------------------------------------
-# for getting the comulative function
+#-------------------------------------------------------------------------------
+#-------------------------------------------------------------------------------
+# for getting the cumulative function
 get.the.p.function <- function(object, ...)
 {
 if (!is.gamlss(object))  stop(paste("This is not an gamlss object", "\n", ""))
@@ -131,7 +134,9 @@ if (!is.gamlss(object))  stop(paste("This is not an gamlss object", "\n", ""))
              else eval(call(dfun,q=object$y, mu=fitted(object,"mu"), sigma=fitted(object,"sigma"), nu=fitted(object,"nu"), tau=fitted(object,"tau")))})
 pfun 
 }             
-#---------------------------------------------------------------------------------------=
+#-------------------------------------------------------------------------------
+#-------------------------------------------------------------------------------
+#-------------------------------------------------------------------------------
 ## The gamlssNP starts here 
 #library(gamlss)
   mixture <- match.arg(mixture)
@@ -142,9 +147,9 @@ pfun
  plot.opt <- control$plot.opt
      damp <- control$damp
 #----
- #gamlss.bi.list <- c("BI", "Binomial", "BB", "Beta Binomial") # binomial denominators
+#gamlss.bi.list <- c("BI", "Binomial", "BB", "Beta Binomial") # binomial denominators
      call <- match.call()
-# we need the data so we can expand them
+# we need the data so we can expand them ---------------------------------------
  if (missing(data)) stop("the data argument is needed")
  if(!missing(data) & any(is.na(data))) 
                stop("The data contains NA's, use data = na.omit(mydata)") 
@@ -162,15 +167,15 @@ pfun
 ## Expand the response
     datak <- expand.vc(data,K)# expand data.
    kindex <- rep(1:K,rep(N,K))# index for the mixtures
-      tmp <- gqz(K, minweight = 1e-14)
-       z0 <- -tmp$l
-        z <- rep(-tmp$l, rep(N, K))
+     tmp1 <- gqz(K, minweight = 1e-14)
+       z0 <- -tmp1$location
+        z <- rep(-tmp1$location, rep(N, K))
  #   p <- tmp$w
  #    tmp <- hermite(K)# grab weights and abscissas
  #     z0 <- tmp$z    # for GQ - masspoints
  #      z <- rep(tmp$z,rep(N,K))
   #pweights <- rep(pweights,K)
-       p <- tmp$w
+       p <- tmp1$weight
    rform <- random
 ##  Generate the random design matrix and append to fixed matrix
    mform <- strsplit(as.character(random)[2],'\\|')[[1]]
@@ -263,11 +268,11 @@ Eta <- fitout$mu.lp + sz
    ML.dev <- ML.dev0 <- deviance(fitout)
      iter <- ml <- 1
 converged <- FALSE
-
+                                                                                
  while (iter <= EMn.cyc && (!converged || (iter<=9 && mixture=='np'  )))
   {
   ##########Start of EM ##############################
-  if (verbose){ cat(iter,"..") }
+  if (verbose){ if (iter%%17==16) cat(iter, "..\n") else cat(iter,"..")  }
 # the gamlss fitting  
 fitout <- if (iter==1)
           { gamlss(formula, family=family, weights=ww*pweights, data = datak, control=g.control, ...)}
@@ -365,7 +370,7 @@ fitout <- if (iter==1)
 #      }  
     if (mixture=="np")
      {
-      ylim<-c(min(followmass[,]),max(followmass[,]))
+      ylim<-c(min(followmass[,], na.rm = TRUE),max(followmass[,], na.rm = TRUE))
        if (plot.opt==2|| plot.opt==3)
          {
             plot(0:(iter-1),followmass[,1],col=1,type='l',ylim=ylim,ylab='mass points',xlab='EM iterations')
@@ -379,10 +384,12 @@ fitout <- if (iter==1)
 ## 
 ## residuals ----
  # there is a problem with binomial also for "qp" prob is fixed 
+              
                prob <- apply(matrix(ww,ncol=K),2,mean)
                  WF <- matrix(get.the.p.function(fitout), ncol=K, nrow=N) 
                  for (i in 1:K)     WF[,i] <- prob[i]*WF[,i] 
                 res <- qnorm(rowSums(WF))
+#               res2 <- apply(ww*matrix(fitout$mu.lp,N,K,byrow=FALSE),1,sum) 
 #get.the.p.function(fitout)
 #WF <- matrix(0, ncol=K, nrow=N)
 # for (i in 1:K)     WF[,i] <- prob[i]*get.the.p.function(allModels[[i]]) 
@@ -409,7 +416,7 @@ fitout$allresiduals <- fitout$residuals
            fitout$N <- N  
            fitout$K <- K
        fitout$type  <- "Mixture"  
-  if (mixture=="np") 
+ if (mixture=="np") 
     {
       if (mform1=='1')
        {
@@ -431,20 +438,24 @@ fitout$allresiduals <- fitout$residuals
          masses     <- prob 
      names(masses)  <- paste('MASS',1:K,sep='')
     fitout$prob     <- masses
-    fitout$family   <- paste( fitout$family, "Mixture with NP")             
+ fitout$orig.family <- fitout$family
+     fitout$family <- paste( fitout$family, "Mixture with NP")             
    #   class(fitout) <- list("gamlssNP", "gamlss")
     } 
  else 
     {
- fitout$mass.points <- fitout$coef[1]+fitout$coef[np]*z0
-    fitout$family   <- paste( fitout$family[[1]], "Mixture with NO")           
-        fitout$prob <- list(gqz(K)$w)          
+  fitout$mass.points <- fitout$coef[1]+fitout$coef[np]*z0
+  fitout$orig.family <- fitout$family
+     fitout$family   <- paste( fitout$family[[1]], "Mixture with NO")           
+         fitout$prob <- list(tmp1$weight)          
    #   class(fitout) <- list("gamlssNO", "gamlss")   
     }
       class(fitout) <- list("gamlssNP", "gamlss")
   fitout
 }
-#----------------------------------------------------------------------------------------
+#-------------------------------------------------------------------------------
+#-------------------------------------------------------------------------------
+#-------------------------------------------------------------------------------
 #get.log.likelihood <- function(obj,y,mu, ...)
 #{
 #if (!is.gamlss(obj))  stop(paste("This is not an gamlss object", "\n", ""))
@@ -459,9 +470,9 @@ fitout$allresiduals <- fitout$residuals
 #    {lik <- eval(call(dfun,y=y, mu=mu, sigma=fitted(obj,"sigma"), nu=fitted(obj,"nu"), tau=fitted(obj,"tau"),log=TRUE))})
 #lik
 #}             
-#----------------------------------------------------------------------------------------
-#----------------------------------------------------------------------------------------
-#----------------------------------------------------------------------------------------
+#-------------------------------------------------------------------------------
+#-------------------------------------------------------------------------------
+#-------------------------------------------------------------------------------
 NP.control <- function(EMcc = 0.001, EMn.cyc = 200, damp = TRUE, trace = TRUE, plot.opt = 3, ...)
 {
         if(EMcc <= 0) {
@@ -477,17 +488,15 @@ warning("the value of no cycles supplied is zero or negative the default value o
               }    
         list(EMcc = EMcc, EMn.cyc = EMn.cyc, trace = as.logical(trace)[1], damp=as.logical(damp)[1], plot.opt=plot.opt)
 }
-#----------------------------------------------------------------------------------------
-#----------------------------------------------------------------------------------------
-#####################################################################################
-#----------------------------------------------------------------------------------------
+#-------------------------------------------------------------------------------
+#-------------------------------------------------------------------------------
+################################################################################
+#-------------------------------------------------------------------------------
  # Auxiliary functions:
-#----------------------------------------------------------------------------------------
-#----------------------------------------------------------------------------------------
-
-#----------------------------------------------------------------------------------------
-
-#----------------------------------------------------------------------------------------
+#-------------------------------------------------------------------------------
+#-------------------------------------------------------------------------------
+#-------------------------------------------------------------------------------
+#-------------------------------------------------------------------------------
 print.gamlssNP <- function (x, digits = max(3, getOption("digits") - 3), ...) 
 {
     K <- x$K 
@@ -523,29 +532,28 @@ print.gamlssNP <- function (x, digits = max(3, getOption("digits") - 3), ...)
         format(signif(x$sbc)), "\n")
     invisible(x)
 }
-#----------------------------------------------------------------------------------------
-#----------------------------------------------------------------------------------------
+#-------------------------------------------------------------------------------
+#-------------------------------------------------------------------------------
+# This need attention MS
 fitted.gamlssNP<-function (object, K=1, what = c("mu", "sigma", "nu", "tau"), ... ) 
-{
+{  
+  what <- match.arg(what)
 if (K%in%seq(1:object$K))
  {
- what <- match.arg(what)
  index <- seq(1+(K-1)*object$N, K*object$N) 
      x <- object[[paste(what,"fv",sep=".")]][index] #fitted(object$mu.fv, ...)[index]
  }
 else 
-  {
-   WF <- matrix(object$mu.fv, ncol=length(object$prob), nrow=object$N)
-   for (i in 1:length(object$prob))    WF[,i] <- object$prob[i]*WF[,i]  #
-    x <-rowSums(WF)
+  { # ?? THIS IS NOT WORKING  MS 
+   x <-  object[[paste(what,"fv",sep=".")]]
   }
  x
 }
-#----------------------------------------------------------------------------------------
+#-------------------------------------------------------------------------------
 family.gamlssNP <- function(object, ...) {
      object$family
  }
-#----------------------------------------------------------------------------------------
+#-------------------------------------------------------------------------------
 # this is the plot.gamlss function 
 # created by PA  May 2002
 # last change by MS Friday, Wednesday, December 17, 2003 at 09:11
@@ -599,7 +607,7 @@ plot.gamlssNP<- function (x, xvar=NULL, parameters=NULL, ts=FALSE, summaries=TRU
          frame.plot = TRUE, 
          col="black", 
          lwd=0.4 ) #col="deepskyblue4", col="darkgreen", 
-         rug(residx, col="red", points(par(col="blue4")))
+         rug(residx, col="red")
  
     qqnorm(residx, main = "Normal Q-Q Plot",
             xlab = "Theoretical Quantiles",
@@ -622,7 +630,7 @@ plot.gamlssNP<- function (x, xvar=NULL, parameters=NULL, ts=FALSE, summaries=TRU
                     b.1 <- m.3^2/m.2^3
                 sqrtb.1 <- sign(m.3)*sqrt(abs(b.1))
                     b.2 <- m.4/m.2^2 
-                     cat("*******************************************************************")
+              cat("*******************************************************************")
                      cat("\n")
                      if (identical(x$type,"Continuous")) 
                          {cat("\t","     Summary of the Quantile Residuals")}
@@ -633,23 +641,23 @@ plot.gamlssNP<- function (x, xvar=NULL, parameters=NULL, ts=FALSE, summaries=TRU
                      cat("               coef. of skewness  = ", sqrtb.1, "\n")
                      cat("               coef. of kurtosis  = ", b.2, "\n")
                      cat("Filliben correlation coefficient  = ", Filliben, "\n")
-                     cat("*******************************************************************")
+              cat("*******************************************************************")
                      cat("\n")
 
                }   
     par(op)
 }
-#----------------------------------------------------------------------------------------
+#-------------------------------------------------------------------------------
 residuals.gamlssNP <- function(object,...)
 {
 object$residuals
 }
-#----------------------------------------------------------------------------------------
+#-------------------------------------------------------------------------------
 #par(mfrow=c(2,2), mar=par("mar")+c(0,1,0,0), col.axis="blue4", col="blue4", col.main="blue4",col.lab="blue4",pch="+",cex=.45, cex.lab=1.2, cex.axis=1, cex.main=1.2  )
-#---------------------------------------------------------------------------------------
-#----------------------------------------------------------------------------------------
+#-------------------------------------------------------------------------------
+#-------------------------------------------------------------------------------
 #
-#----------------------------------------------------------------------------------------
+#-------------------------------------------------------------------------------
 # this is  Nick Sofroniou function from the package 
 # npmlreg of  Jochen Einbeck, Ross Darnell and John Hinde (2006).
  gqz <- function (numnodes = 20, minweight = 1e-06) 
@@ -663,7 +671,7 @@ object$residuals
     names(h) <- c("location", "weight")
     h
 }
-#----------------------------------------------------------------------------------------
+#-------------------------------------------------------------------------------
 # this  Gordon Smyth function from package 
 # package:statmod 
 # see there for documantation
@@ -727,4 +735,4 @@ gauss.quad <- function (n, kind = "legendre", alpha = 0, beta = 0)
     x <- rev(vd$values)
     list(nodes = x, weights = w)
 }
-#----------------------------------------------------------------------------------------
+#-------------------------------------------------------------------------------
